"""
This is a package that allows you to perform calculations involving quantities with known or estimated uncertainties.

It exposes the class Measurement, which is a numeric-like class that can be used with common arithmetic operations.

This package also includes a way to parse files (through create_from_file), giving the corresponding mean and sigma.
"""